import { EditorView } from '@codemirror/view';
import { Compartment } from '@codemirror/state';
import { shikiToCodeMirror, updateEffect, themeCompartment } from '@cmshiki/shiki';

function partitionOptions(options) {
  const ShikiKeys = [
    "lang",
    "langAlias",
    "theme",
    "themes",
    "themeStyle",
    "includeExplanation",
    "cssVariablePrefix",
    "colorReplacements",
    "warnings",
    "tokenizeMaxLineLength",
    "tokenizeTimeLimit",
    "defaultColor"
  ];
  const CodeMirrorKeys = [
    "extensions",
    "parent",
    "state",
    "selection",
    "dispatch",
    "dispatchTransactions",
    "root",
    "scrollTo",
    "doc",
    "onUpdate"
  ];
  function filter(keys) {
    return Object.fromEntries(Object.entries(options).filter(([key]) => keys.includes(key)));
  }
  return {
    shikiOptions: filter(ShikiKeys),
    CodeMirrorOptions: filter(CodeMirrorKeys)
  };
}

var __defProp = Object.defineProperty;
var __defNormalProp = (obj, key, value) => key in obj ? __defProp(obj, key, { enumerable: true, configurable: true, writable: true, value }) : obj[key] = value;
var __publicField = (obj, key, value) => {
  __defNormalProp(obj, typeof key !== "symbol" ? key + "" : key, value);
  return value;
};
const shikiComp = new Compartment();
class ShikiEditor {
  constructor(options) {
    this.options = options;
    __publicField(this, "view");
    __publicField(this, "getTheme");
    const { shikiOptions, CodeMirrorOptions: cmOptions } = partitionOptions(options);
    this.view = new EditorView({
      ...cmOptions,
      extensions: Array.isArray(cmOptions.extensions) ? cmOptions.extensions.concat(shikiComp.of([])) : void 0
    });
    this.getTheme = this.registerInternal(shikiOptions).then(({ getTheme, extensions }) => {
      this.view.dispatch({
        effects: shikiComp.reconfigure(extensions)
      });
      return getTheme;
    });
  }
  async registerInternal(options) {
    const { shiki, getTheme } = await shikiToCodeMirror(options);
    return {
      getTheme,
      extensions: [
        EditorView.updateListener.of((u) => {
          if (u.docChanged) {
            options.onUpdate?.(u);
          }
        }),
        shiki
      ]
    };
  }
  /**
   * Dynamically set listening events,listen when the document changed.
   * @param {(u: ViewUpdate)}
   */
  setOnUpdate(callback) {
    this.options.onUpdate = callback;
  }
  /**
   * Dynamically set listening events,listen when the document changed.
   * @param {(u: ViewUpdate)}
   */
  destroy() {
    this.view.destroy();
  }
  update(options) {
    this.view.dispatch({
      effects: updateEffect.of(options)
    });
  }
  /**
   * change theme
   * 
   * @param {string} name - theme style name,like `light`、`dark`、`dim`
   */
  async changeTheme(name) {
    this.view.dispatch({
      effects: themeCompartment.reconfigure((await this.getTheme)(name, this.view))
    });
  }
  getValue() {
    return this.view.state.doc.toString();
  }
  setValue(newCode) {
    this.view.dispatch({
      changes: { from: 0, to: this.view.state.doc.length, insert: newCode }
    });
  }
}

export { ShikiEditor };
